package com.rnupdate;


import android.widget.Toast;

import com.facebook.react.bridge.Callback;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.rnupdate.updateUtils.Constants;
import com.rnupdate.updateUtils.UpdateChecker;
import com.rnupdate.updatepluginlib.UpdateBuilder;
import com.rnupdate.updatepluginlib.UpdateConfig;
import com.rnupdate.updatepluginlib.callback.UpdateCheckCB;
import com.rnupdate.updatepluginlib.callback.UpdateDownloadCB;
import com.rnupdate.updatepluginlib.model.Update;
import com.rnupdate.updatepluginlib.model.UpdateParser;
import com.rnupdate.updatepluginlib.strategy.UpdateStrategy;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.File;


/**
 * Created by Kun on 16/4/12.
 */
public class UpdateModule extends ReactContextBaseJavaModule {
    public UpdateModule(ReactApplicationContext reactContext) {
        super(reactContext);
    }
    public static String apkMd5 = "";
    @Override
    public String getName() {
        return "UpdateModule";
    }
    @ReactMethod
    public void checkUpdate(String url,String type,boolean enable,Callback cb){
        UpdateChecker.getInstance().CheckerForUpdate(getCurrentActivity(),url,type,cb,enable);
    }

    @ReactMethod//显示更新提醒;自动安装(显示安装提醒为true时此处无效);显示安装提醒;显示下载进度
    public void checkWork(String url, final boolean isShowUpdateDialog,
                          final boolean isAutoInstall, final boolean isShowInstallDialog,
                          final boolean isShowDownloadDialog, final Callback callback){
        UpdateConfig.getConfig()
                // 必填：数据更新接口
                .url(url)
                        // 必填：用于从数据更新接口获取的数据response中。解析出Update实例。以便框架内部处理
                .jsonParser(new UpdateParser() {
                    @Override
                    public Update parse(String response) {
                        //Log.e("Module",response);
                        // 此处一个Update对象
                        Update update = new Update(response);
                        // 此apk包的更新时间
                        update.setUpdateTime(System.currentTimeMillis());
                        try {
                            JSONObject obj = new JSONObject(response);
                            final String updateMessage = obj.getString(Constants.APK_UPDATE_CONTENT);
                            final String apkUrl = obj.getString(Constants.APK_DOWNLOAD_URL);
                            int apkCode = obj.getInt(Constants.APK_VERSION_CODE);
                            String apkname = obj.getString(Constants.APK_VERSION_NAME);
                            apkMd5 = obj.getString(Constants.APK_CHECK_MD5);
                            // 此apk包的下载地址
                            update.setUpdateUrl(apkUrl);
                            // 此apk包的版本号
                            update.setVersionCode(apkCode);
                            // 此apk包的版本名称
                            update.setVersionName(apkname);
                            // 此apk包的更新内容
                            update.setUpdateContent(updateMessage);
                            // 此apk包是否为强制更新
                            update.setForced(false);
                            // 是否忽略此次版本更新
                            update.setIgnore(false);
                        } catch (JSONException e) {
                            e.printStackTrace();
                        }
                        return update;
                    }
                });
        final int[] cancelIndex = {1};
        UpdateBuilder.create().strategy(new UpdateStrategy() {
            @Override
            public boolean isShowUpdateDialog(Update update) {
                return isShowUpdateDialog;
            }

            @Override
            public boolean isAutoInstall() {
                return isAutoInstall;
            }

            @Override
            public boolean isShowInstallDialog() {
                return isShowInstallDialog;
            }

            @Override
            public boolean isShowDownloadDialog() {
                return isShowDownloadDialog;
            }
        }).downloadCB(new UpdateDownloadCB() {
            @Override
            public void onUpdateStart() {
                // 下载开始
            }

            @Override
            public void onUpdateComplete(File file) {
                // 下载完成
                cancelIndex[0] = 2;
            }

            @Override
            public void onUpdateProgress(long current, long total) {
                // 下载进度
            }

            @Override
            public void onUpdateError(int code, String errorMsg) {
                // 下载apk错误
                callback.invoke(code+":"+errorMsg);
            }
        }).checkCB(new UpdateCheckCB() {
            @Override
            public void hasUpdate(Update update) {
                // 有新版本APK更新的回调
            }

            @Override
            public void noUpdate() {
                // 没有新版本的回调
                Toast.makeText(getCurrentActivity(), "当前为最新版,感谢您的使用", Toast.LENGTH_SHORT).show();
                callback.invoke(0);
            }

            @Override
            public void onCheckError(int i, String s) {
                // 检查错误的回调
                callback.invoke(-1);
            }

            @Override
            public void onUserCancel() {
                // 用于取消更新时的回调
                Toast.makeText(getCurrentActivity(), "您取消了本次更新", Toast.LENGTH_SHORT).show();
                callback.invoke(cancelIndex[0]);
            }
        }).check(getCurrentActivity());
    }
}
