package com.rnupdate.updateUtils;


import android.app.Activity;
import android.app.ActivityManager;
import android.app.AlertDialog.Builder;
import android.app.Dialog;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Looper;
import androidx.core.app.NotificationCompat;
import android.util.Log;
import android.widget.Toast;

import com.facebook.react.bridge.Callback;
import com.rnupdate.UpdateModule;

import org.json.JSONObject;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.List;

/**
 * Created by Kun on 16/4/7.
 */
public class UpdateChecker {
    private static UpdateChecker instance;
    public static synchronized UpdateChecker getInstance() {
        if (instance == null) {
            instance = new UpdateChecker();
        }
        return instance;
    }

    private static final String TAG = "UpdateChecker";
    private Thread mThread;

    private boolean enable;//true为主动需提示
    public void CheckerForUpdate(final Context context, final String url, final String type, final Callback cb, final boolean enable) {
        this.enable = enable;
        mThread = new Thread() {
            @Override
            public void run() {
                //默认有更新
                //showNotification(MyApplication.getContextObject(), "", "");
                //通过服务器数据判断
                //String result = check(context,url);
                String result = null;
                try {
                    result = readParse(url);
                } catch (Exception e) {
                    if (enable){
                    ((Activity)context).runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            Toast.makeText(context, "地址不能正常解析", Toast.LENGTH_SHORT).show();
                        }
                    });}
                    cb.invoke(5+":"+e.getMessage());
                    e.printStackTrace();
                    return;
                }
                //{"url":"http:\/\/www.tuofeng.cn\/resources\/JiJiaApp-ADEF-1.0.12-armeabi-v7a-release-20160302144441.apk","updateMessage":"测试内容","versionCode":"12"}
                //result = "{\"url\":\"http://www.tuofeng.cn/resources/JiJiaApp-ADEF-1.0.12-armeabi-v7a-release-20160302144441.apk\",\"updateMessage\":\"测试内容\",\"versionCode\":\"12\"}";
                if (result != null) {
                    //信息不为空
                    parseJson(context, result,type,cb);
                } else {
                    if (enable){
                    ((Activity)context).runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            Toast.makeText(context, "信息获取异常 请重试", Toast.LENGTH_SHORT).show();
                        }
                    });}
                    cb.invoke(4);
                }

            }
        };
        mThread.start();
    }
    //访问请求检测地址获取最新的产品消息
    protected   String readParse(String urlPath) throws Exception {
        ByteArrayOutputStream outStream = new ByteArrayOutputStream();
        byte[] data = new byte[1024];
        int len = 0;
        URL url = new URL(urlPath);
        HttpURLConnection conn = (HttpURLConnection) url.openConnection();
        InputStream inStream = conn.getInputStream();
        while ((len = inStream.read(data)) != -1) {
            outStream.write(data, 0, len);
        }
        inStream.close();
        return new String(outStream.toByteArray());//通过out.Stream.toByteArray获取到写的数据
    }

    //解析获取的信息
    private void parseJson(final Context context, String json, String type,Callback cb) {
        mThread.interrupt();
        Looper.prepare();
        try {
            JSONObject obj = new JSONObject(json);
            final String updateMessage = obj.getString(Constants.APK_UPDATE_CONTENT);
            final String apkUrl = obj.getString(Constants.APK_DOWNLOAD_URL);
            int apkCode = obj.getInt(Constants.APK_VERSION_CODE);
            UpdateModule.apkMd5 = obj.getString(Constants.APK_CHECK_MD5);
            //Log.e(TAG,"updateMessage="+updateMessage+"versionCode"+apkCode);
            //当前运行的代码版本号
            int versionCode = context.getPackageManager().getPackageInfo(context.getPackageName(), 0).versionCode;

            if (apkCode > versionCode) {
                //当前代码的版本号较低时，状态栏进行提示更新
                if (type.equals("1")){
                    ((Activity)context).runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            showNoticeDialog(context, updateMessage, apkUrl);
                        }
                    });

                }else {

                        ((Activity)context).runOnUiThread(new Runnable() {
                            @Override
                            public void run() {
                                Toast.makeText(context, "新版发布 详情请查看通知栏", Toast.LENGTH_SHORT).show();
                            }
                        });

                    showNotification(context, updateMessage, apkUrl);
                }
                cb.invoke(1);
            }else {
                if (enable){
                ((Activity)context).runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        Toast.makeText(context, "当前已是最新版本 感谢您的使用", Toast.LENGTH_SHORT).show();
                    }
                });}
                cb.invoke(0);
            }

        }  catch (Exception e){
            cb.invoke(3+":"+e.getMessage());
        }
    }

    /**
     * Show Notification
     */
    public void showNotification(Context context, String content, String apkUrl) {
        android.app.Notification noti;
        Intent myIntent = new Intent(context, DownloadService.class);
        myIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        myIntent.putExtra(Constants.APK_DOWNLOAD_URL, apkUrl);
        PendingIntent pendingIntent = PendingIntent.getService(context, 0, myIntent, PendingIntent.FLAG_UPDATE_CURRENT);
        int smallIcon = context.getApplicationInfo().icon;
        //设置通知的<通知显示文字><通知的标题><通知的内容><通知的图标><点击效果>
        noti = new NotificationCompat.Builder(context).setTicker("发现新版本")
                .setContentTitle("发现新版本").setContentText(content).setSmallIcon(smallIcon)
                .setContentIntent(pendingIntent).build();

        noti.flags = android.app.Notification.FLAG_AUTO_CANCEL;
        NotificationManager notificationManager = (NotificationManager) context.getSystemService(Context.NOTIFICATION_SERVICE);
        notificationManager.notify(0, noti);
    }
    /**
     * Show Notification
     */
    public void showNoticeDialog(final Context context, String content, final String apkUrl) {
        Builder builder = new Builder(context);
        builder.setTitle("发现新版本");
        builder.setMessage(content);
        builder.setPositiveButton("立即更新", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                Intent myIntent = new Intent(context, DownloadService.class);
                myIntent.putExtra(Constants.APK_DOWNLOAD_URL, apkUrl);
                context.startService(myIntent);
                dialog.dismiss();
            }
        });
        builder.setNegativeButton("稍后提醒", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                dialog.dismiss();
            }
        });
        Dialog noticeDialog = builder.create();
        if (isBackground(context))
            noticeDialog.show();
    }

    public boolean isBackground(Context context) {
        ActivityManager activityManager = (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
        List<ActivityManager.RunningAppProcessInfo> appProcesses = activityManager.getRunningAppProcesses();
        if (appProcesses == null) return false;
        for (ActivityManager.RunningAppProcessInfo appProcess : appProcesses) {
            if (appProcess.processName.equals(context.getPackageName())) {
                if (appProcess.importance == ActivityManager.RunningAppProcessInfo.IMPORTANCE_FOREGROUND) {
                    // Log.i("前台", appProcess.processName);
                    return true;
                }else{
                    // Log.i("后台", appProcess.processName);
                    return false;
                }
            }
        }
        return false;
    }


}
